/**
 * @author Damian_Dziedzic_&_Mateusz_Koacz
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Class XsdSchemaValidator validating xml with xsd choosen in window.
 */
class XsdSchemaValidator {
	private static int error_Count = 0;
	protected static List<String> list = new LinkedList<String>();
	protected static String namespace;
	protected static String targetNamespace;
	private ResourceResolver rs = new ResourceResolver();

	/**
	 * Method responsible for validating and parsing xml and xsd
	 * 
	 * @param xml
	 *            contain the File with xml
	 * @param schemaName
	 *            contain path to xsd file
	 * @throws Exception
	 *             throws Exceprions when files are parsing, validating etc...
	 */
	void validate(File xml, String schemaName) throws Exception {
		/**
		 * Create validator from schema and add him our exception handler
		 */
		Validator validator = sch(schemaName).newValidator();
		validator.setErrorHandler(new MyErrorHandler());
		/**
		 * validation xml against choosen xsd
		 */
		validator.validate(new DOMSource(doc(xml)));

	}

	/**
	 * Method returning list of errors if validator find some.
	 * 
	 * @return String made from list of errors
	 */
	public String getList() {
		String s = "";
		for (int i = 0; i < list.size(); i++) {
			s += list.get(i);
		}
		return s;
	}

	public String getNamespace() {
		return namespace;
	}

	public String getTargetNamespace() {
		return targetNamespace;
	}

	/**
	 * Method to get value from class XsdSchemaValidator
	 * 
	 * @return count of errors accrued in validation
	 */
	public int error_Count() {
		return error_Count;
	}

	/**
	 * Method to create document using DOM from xml file
	 * 
	 * @param xml
	 *            File with xml
	 * @return document parsed by DOM
	 * @throws Exception
	 *             throws by parser etc...
	 */
	public Document doc(File xml) throws Exception {
		/**
		 * Create DocumentBuldierFactory and set all necessary parameters
		 */
		DocumentBuilderFactory builderFactory = DocumentBuilderFactory
				.newInstance();
		/**
		 * .setValidating(flase) - to not validating xml when parsing, (if set
		 * true, parser will search schemas online and try validate xml)
		 * .setNamespaceAware(true) - set support xml namespaces .setCoalescing
		 * - i don't remember right now why I set that up, but it must be
		 * true...
		 */
		builderFactory.setValidating(false);
		builderFactory.setNamespaceAware(true);
		builderFactory.setCoalescing(true);
		/**
		 * Create xml Parser
		 */
		DocumentBuilder parser = builderFactory.newDocumentBuilder();
		/**
		 * Create Stream with text from xml file
		 */
		InputStream is = new FileInputStream(xml.getPath());
		/**
		 * Parsing xml
		 */
		Document document = parser.parse(is);
		return document;
	}

	/**
	 * Method to create schema
	 * 
	 * @param schemaName
	 *            contain path to schema file
	 * @return Complete schema in memory
	 * @throws Exception
	 *             throws by Schema
	 */
	public Schema sch(String schemaName) throws Exception {
		/**
		 * Creating new SchemaFactory instance
		 */
		SchemaFactory factory = SchemaFactory
				.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		/**
		 * ResourceResolver set him to search included and
		 * importer files in folder from which XSD was choosen
		 */
		
		rs.setPath(schemaName);
		/**
		 * Set SchemaFactory to use our ResourceResolver
		 */
		factory.setResourceResolver(rs);
		/**
		 * Choose file which contain xsd
		 */
		Source schemaFile = new StreamSource(schemaName);
		/**
		 * Create in memory schema from source file if contain imports or
		 * includes, ResourceResolver will find add them to schema
		 */
		try {
			Schema schema = factory.newSchema(schemaFile);
			namespace = rs.getNamespace().get(0);
			targetNamespace = rs.getTargetNameSpace().get(0);
			return schema;
		} catch (SAXException e) {
			list = rs.IOEx();
			throw new MyException();
		}
		

	}
	public void clear(){
		list.clear();
		rs.clear();
		error_Count = 0;
	}

	/**
	 * Inner class MyErrorHandler implements ErrorHandler to handle errors which
	 * may occur
	 */
	private static class MyErrorHandler implements ErrorHandler {
		public void warning(SAXParseException e) throws SAXException {
			error_Count++;
			list.add("Warning: \n");
			list.add("   Message: " + e.getMessage() + "\n");
		}

		public void error(SAXParseException e) throws SAXException {
			error_Count++;
			list.add("Error: \n");
			list.add("   Message: " + e.getMessage() + "\n");
		}

		public void fatalError(SAXParseException e) throws SAXException {
			error_Count++;
			list.add("Fatal error: \n");
			list.add("   Message: " + e.getMessage() + "\n");
		}

	}
}
